<?php

namespace App\DataProvider;

use Doctrine\Persistence\ManagerRegistry;
use ApiPlatform\Core\DataProvider\CollectionDataProviderInterface;
use ApiPlatform\Core\DataProvider\RestrictedDataProviderInterface;
use App\Entity\PriceNow;
use App\Entity\Prices;

class PriceNowProvider implements CollectionDataProviderInterface, RestrictedDataProviderInterface
{
    private $doctrine;

    public function __construct(ManagerRegistry $doctrine)
    {
        $this->doctrine = $doctrine;
    }

    private function getPriceToday(): ?Prices {
        $em = $this->doctrine->getManager();
        return $em->getRepository(Prices::class)->findOneBy(array(
            'day' => new \DateTime('today')
        ));
    }
    private function setPriceToday(Prices $price):bool  {
        $em = $this->doctrine->getManager();
        $em->persist($price);
        $em->flush();
        return true;
    }


    private function getPriceHour(string $hh, Prices $price):float 
    {
        switch($hh) {
            case '00': return $price->h00;
            case '01': return $price->h01;
            case '02': return $price->h02;
            case '03': return $price->h03;
            case '04': return $price->h04;
            case '05': return $price->h05;
            case '06': return $price->h06;
            case '07': return $price->h07;
            case '08': return $price->h08;
            case '09': return $price->h09;
            case '10': return $price->h10;
            case '11': return $price->h11;
            case '12': return $price->h12;
            case '13': return $price->h13;
            case '14': return $price->h14;
            case '15': return $price->h15;
            case '16': return $price->h16;
            case '17': return $price->h17;
            case '18': return $price->h18;
            case '19': return $price->h19;
            case '20': return $price->h20;
            case '21': return $price->h21;
            case '22': return $price->h22;
            case '23': return $price->h23;
        }

        return -1;
    }


    private function setPriceHour(string $hh, float $price, Prices $priceEntity):void
    {
        switch($hh) {
            case '00': $priceEntity->h00 = $price; break;
            case '01': $priceEntity->h01 = $price; break;
            case '02': $priceEntity->h02 = $price; break;
            case '03': $priceEntity->h03 = $price; break;
            case '04': $priceEntity->h04 = $price; break;
            case '05': $priceEntity->h05 = $price; break;
            case '06': $priceEntity->h06 = $price; break;
            case '07': $priceEntity->h07 = $price; break;
            case '08': $priceEntity->h08 = $price; break;
            case '09': $priceEntity->h09 = $price; break;
            case '10': $priceEntity->h10 = $price; break;
            case '11': $priceEntity->h11 = $price; break;
            case '12': $priceEntity->h12 = $price; break;
            case '13': $priceEntity->h13 = $price; break;
            case '14': $priceEntity->h14 = $price; break;
            case '15': $priceEntity->h15 = $price; break;
            case '16': $priceEntity->h16 = $price; break;
            case '17': $priceEntity->h17 = $price; break;
            case '18': $priceEntity->h18 = $price; break;
            case '19': $priceEntity->h19 = $price; break;
            case '20': $priceEntity->h20 = $price; break;
            case '21': $priceEntity->h21 = $price; break;
            case '22': $priceEntity->h22 = $price; break;
            case '23': $priceEntity->h23 = $price; break;
        }
    }


    public function getCollection(string $resourceClass, string $operationName = null) : iterable
    {
        // Get the today's price (from cache)
        /*
        $em = $this->doctrine->getManager();
        $dql = "SELECT hour_start, hour_end, price FROM prices WHERE day = CURRENT_DATE()";
		$query = $em->getConnection()->prepare($dql);
        $prices = $query->executeQuery()->fetchAllAssociative();
        */
        $priceToday = $this->getPriceToday();

        if ( is_null($priceToday) ) {
            // Get the today's price (from internet)
            $url = 'https://api.esios.ree.es/archives/70/download_json?locale=es&date='.date('Y-m-d'); 
            $data = file_get_contents($url);
            $json = json_decode($data, true);

            if ( !array_key_exists('PVPC', $json) ) 
                throw new Exception('Unable to get the prices for today');

            $priceToday = new Prices();
            $ddMMyyy = explode('/',$json['PVPC'][0]['Dia']);
            $priceToday->day = new \DateTime($ddMMyyy[2].'/'.$ddMMyyy[1].'/'.$ddMMyyy[0].' 00:00:00');
            foreach( $json['PVPC'] as $pvpc ) {
                $hh = explode('-',$pvpc['Hora'])[0];
                $price = str_replace(',','.',$pvpc['PCB']) / 1000;

                $this->setPriceHour($hh, $price, $priceToday);
            }
            $this->setPriceToday($priceToday);
        }

        $priceNow = new PriceNow();
        $priceNow->id = $priceToday->getId();
        $priceNow->min = min($priceToday->h00,$priceToday->h01,$priceToday->h02,$priceToday->h03,$priceToday->h04,$priceToday->h05,$priceToday->h06,$priceToday->h07,$priceToday->h08,$priceToday->h09,$priceToday->h10,$priceToday->h11,$priceToday->h12,$priceToday->h13,$priceToday->h14,$priceToday->h15,$priceToday->h16,$priceToday->h17,$priceToday->h18,$priceToday->h19,$priceToday->h20,$priceToday->h21,$priceToday->h22,$priceToday->h23);
        $priceNow->max = max($priceToday->h00,$priceToday->h01,$priceToday->h02,$priceToday->h03,$priceToday->h04,$priceToday->h05,$priceToday->h06,$priceToday->h07,$priceToday->h08,$priceToday->h09,$priceToday->h10,$priceToday->h11,$priceToday->h12,$priceToday->h13,$priceToday->h14,$priceToday->h15,$priceToday->h16,$priceToday->h17,$priceToday->h18,$priceToday->h19,$priceToday->h20,$priceToday->h21,$priceToday->h22,$priceToday->h23);
        $priceNow->avg = ($priceToday->h00+$priceToday->h01+$priceToday->h02+$priceToday->h03+$priceToday->h04+$priceToday->h05+$priceToday->h06+$priceToday->h07+$priceToday->h08+$priceToday->h09+$priceToday->h10+$priceToday->h11+$priceToday->h12+$priceToday->h13+$priceToday->h14+$priceToday->h15+$priceToday->h16+$priceToday->h17+$priceToday->h18+$priceToday->h19+$priceToday->h20+$priceToday->h21+$priceToday->h22+$priceToday->h23)/24;
        $priceNow->now = $this->getPriceHour( Date('H'), $priceToday);
// print_r($priceNow);exit;
        return [$priceNow];        
    }

    public function supports(string $resourceClass, string $operationName = null, array $context = []): bool
    {
        return $resourceClass === PriceNow::class;
    }
}